/*******************************************************************************
* Copyright 2017-2018 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

#pragma once

#include "ngraph/ops/util/unary_elementwise.hpp"
#include "ngraph/types/type.hpp"

namespace ngraph
{
    namespace op
    {
        /// \brief Elementwise type conversion operation.
        class Convert : public util::UnaryElementwise
        {
        public:
            /// \brief Constructs a conversion operation.
            ///
            /// \param arg          Node that produces the input tensor.
            /// \param element_type Element type for the output tensor.
            Convert(const std::shared_ptr<Node>& arg, const ngraph::element::Type& element_type);

            virtual std::shared_ptr<Node> copy_with_new_args(
                const std::vector<std::shared_ptr<Node>>& new_args) const override
            {
                if (new_args.size() != 1)
                {
                    throw ngraph_error("Incorrect number of new arguments");
                }
                return std::make_shared<Convert>(new_args.at(0), m_element_type);
            }

            const element::Type& get_convert_element_type() const { return m_element_type; }
        protected:
            const ngraph::element::Type m_element_type;
            virtual void generate_adjoints(autodiff::Adjoints& adjoints,
                                           const std::shared_ptr<Node>& delta) override;
        };
    }
}
